@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2008 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -53,11 +53,11 @@ static void g_mime_multipart_init (GMimeMultipart *multipart, GMimeMultipartClas
 static void g_mime_multipart_finalize (GObject *object);
 
 /* GMimeObject class methods */
-static void multipart_init (GMimeObject *object);
-static void multipart_add_header (GMimeObject *object, const char *header, const char *value);
+static void multipart_prepend_header (GMimeObject *object, const char *header, const char *value);
+static void multipart_append_header (GMimeObject *object, const char *header, const char *value);
 static void multipart_set_header (GMimeObject *object, const char *header, const char *value);
 static const char *multipart_get_header (GMimeObject *object, const char *header);
-static void multipart_remove_header (GMimeObject *object, const char *header);
+static gboolean multipart_remove_header (GMimeObject *object, const char *header);
 static void multipart_set_content_type (GMimeObject *object, GMimeContentType *content_type);
 static char *multipart_get_headers (GMimeObject *object);
 static ssize_t multipart_write_to_stream (GMimeObject *object, GMimeStream *stream);
@@ -65,7 +65,7 @@ static ssize_t multipart_write_to_stream (GMimeObject *object, GMimeStream *stre
 /* GMimeMultipart class methods */
 static void multipart_add_part (GMimeMultipart *multipart, GMimeObject *part);
 static void multipart_add_part_at (GMimeMultipart *multipart, GMimeObject *part, int index);
-static void multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part);
+static gboolean multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part);
 static GMimeObject *multipart_remove_part_at (GMimeMultipart *multipart, int index);
 static GMimeObject *multipart_get_part (GMimeMultipart *multipart, int index);
 static int multipart_get_number (GMimeMultipart *multipart);
@@ -111,11 +111,11 @@ g_mime_multipart_class_init (GMimeMultipartClass *klass)
 	
 	gobject_class->finalize = g_mime_multipart_finalize;
 	
-	object_class->init = multipart_init;
-	object_class->add_header = multipart_add_header;
+	object_class->prepend_header = multipart_prepend_header;
+	object_class->append_header = multipart_append_header;
+	object_class->remove_header = multipart_remove_header;
 	object_class->set_header = multipart_set_header;
 	object_class->get_header = multipart_get_header;
-	object_class->remove_header = multipart_remove_header;
 	object_class->set_content_type = multipart_set_content_type;
 	object_class->get_headers = multipart_get_headers;
 	object_class->write_to_stream = multipart_write_to_stream;
@@ -133,51 +133,46 @@ g_mime_multipart_class_init (GMimeMultipartClass *klass)
 static void
 g_mime_multipart_init (GMimeMultipart *multipart, GMimeMultipartClass *klass)
 {
+	multipart->children = g_ptr_array_new ();
 	multipart->boundary = NULL;
 	multipart->preface = NULL;
 	multipart->postface = NULL;
-	multipart->subparts = NULL;
 }
 
 static void
 g_mime_multipart_finalize (GObject *object)
 {
 	GMimeMultipart *multipart = (GMimeMultipart *) object;
-	GList *node;
+	guint i;
 	
 	g_free (multipart->boundary);
 	g_free (multipart->preface);
 	g_free (multipart->postface);
 	
-	node = multipart->subparts;
-	while (node) {
-		GMimeObject *part;
-		
-		part = node->data;
-		g_object_unref (part);
-		node = node->next;
-	}
-	g_list_free (multipart->subparts);
+	for (i = 0; i < multipart->children->len; i++)
+		g_object_unref (multipart->children->pdata[i]);
+	
+	g_ptr_array_free (multipart->children, TRUE);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-
 static void
-multipart_init (GMimeObject *object)
-{
-	/* no-op */
-	GMIME_OBJECT_CLASS (parent_class)->init (object);
-}
-
-static void
-multipart_add_header (GMimeObject *object, const char *header, const char *value)
+multipart_prepend_header (GMimeObject *object, const char *header, const char *value)
 {
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
-	
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
-		GMIME_OBJECT_CLASS (parent_class)->add_header (object, header, value);
+		GMIME_OBJECT_CLASS (parent_class)->prepend_header (object, header, value);
+}
+
+static void
+multipart_append_header (GMimeObject *object, const char *header, const char *value)
+{
+	/* Make sure that the header is a Content-* header, else it
+           doesn't belong on a multipart */
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
+		GMIME_OBJECT_CLASS (parent_class)->append_header (object, header, value);
 }
 
 static void
@@ -189,7 +184,6 @@ multipart_set_header (GMimeObject *object, const char *header, const char *value
 	
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
-	
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
 }
@@ -199,21 +193,21 @@ multipart_get_header (GMimeObject *object, const char *header)
 {
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
-	
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
 	else
 		return NULL;
 }
 
-static void
+static gboolean
 multipart_remove_header (GMimeObject *object, const char *header)
 {
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
+	if (g_ascii_strncasecmp ("Content-", header, 8) != 0)
+		return FALSE;
 	
-	if (!g_ascii_strncasecmp ("Content-", header, 8))
-		GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
+	return GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
 }
 
 static void
@@ -241,17 +235,17 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	GMimeMultipart *multipart = (GMimeMultipart *) object;
 	ssize_t nwritten, total = 0;
 	GMimeObject *part;
-	GList *node;
+	guint i;
 	
 	/* make sure a boundary is set unless we are writing out a raw
 	 * header (in which case it should already be set... or if
 	 * not, then it's a broken multipart and so we don't want to
 	 * alter it or we'll completely break the output) */
-	if (!multipart->boundary && !g_mime_header_has_raw (object->headers))
+	if (!multipart->boundary && !g_mime_header_list_has_raw (object->headers))
 		g_mime_multipart_set_boundary (multipart, NULL);
 	
 	/* write the content headers */
-	if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
+	if ((nwritten = g_mime_header_list_write_to_stream (object->headers, stream)) == -1)
 		return -1;
 	
 	total += nwritten;
@@ -270,9 +264,8 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 		total += nwritten;
 	}
 	
-	node = multipart->subparts;
-	while (node) {
-		part = node->data;
+	for (i = 0; i < multipart->children->len; i++) {
+		part = multipart->children->pdata[i];
 		
 		/* write the boundary */
 		if ((nwritten = g_mime_stream_printf (stream, "\n--%s\n", multipart->boundary)) == -1)
@@ -285,8 +278,6 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 			return -1;
 		
 		total += nwritten;
-		
-		node = node->next;
 	}
 	
 	/* write the end-boundary (but only if a boundary is set) */
@@ -304,7 +295,7 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 		
 		total += nwritten;
 	}
-		
+	
 	return total;
 }
 
@@ -315,7 +306,7 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
  * Creates a new MIME multipart object with a default content-type of
  * multipart/mixed.
  *
- * Returns an empty MIME multipart object with a default content-type of
+ * Returns: an empty MIME multipart object with a default content-type of
  * multipart/mixed.
  **/
 GMimeMultipart *
@@ -340,7 +331,7 @@ g_mime_multipart_new (void)
  * Creates a new MIME multipart object with a content-type of
  * multipart/@subtype.
  *
- * Returns an empty MIME multipart object with a content-type of
+ * Returns: an empty MIME multipart object with a content-type of
  * multipart/@subtype.
  **/
 GMimeMultipart *
@@ -381,7 +372,7 @@ g_mime_multipart_set_preface (GMimeMultipart *multipart, const char *preface)
  *
  * Gets the preface on the multipart.
  *
- * Returns a pointer to the preface string on the multipart.
+ * Returns: a pointer to the preface string on the multipart.
  **/
 const char *
 g_mime_multipart_get_preface (GMimeMultipart *multipart)
@@ -415,7 +406,7 @@ g_mime_multipart_set_postface (GMimeMultipart *multipart, const char *postface)
  *
  * Gets the postface on the multipart.
  *
- * Returns a pointer to the postface string on the multipart.
+ * Returns: a pointer to the postface string on the multipart.
  **/
 const char *
 g_mime_multipart_get_postface (GMimeMultipart *multipart)
@@ -430,7 +421,8 @@ static void
 multipart_add_part (GMimeMultipart *multipart, GMimeObject *part)
 {
 	g_object_ref (part);
-	multipart->subparts = g_list_append (multipart->subparts, part);
+	
+	g_ptr_array_add (multipart->children, part);
 }
 
 
@@ -451,11 +443,32 @@ g_mime_multipart_add_part (GMimeMultipart *multipart, GMimeObject *part)
 }
 
 
+static void
+ptr_array_insert (GPtrArray *array, guint index, gpointer object)
+{
+	unsigned char *dest, *src;
+	guint n;
+	
+	g_ptr_array_set_size (array, array->len + 1);
+	
+	if (index == array->len) {
+		/* need to move items down */
+		dest = ((unsigned char *) array->pdata) + (sizeof (void *) * (index + 1));
+		src = ((unsigned char *) array->pdata) + (sizeof (void *) * index);
+		n = array->len - index - 1;
+		
+		g_memmove (dest, src, (sizeof (void *) * n));
+	}
+	
+	array->pdata[index] = object;
+}
+
 static void
 multipart_add_part_at (GMimeMultipart *multipart, GMimeObject *part, int index)
 {
 	g_object_ref (part);
-	multipart->subparts = g_list_insert (multipart->subparts, part, index);
+	
+	ptr_array_insert (multipart->children, index, part);
 }
 
 
@@ -478,39 +491,15 @@ g_mime_multipart_add_part_at (GMimeMultipart *multipart, GMimeObject *part, int
 }
 
 
-static void
+static gboolean
 multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part)
 {
-	GList *node;
-	
-	/* *sigh* fucking glist... */
-	
-	node = multipart->subparts;
-	while (node) {
-		if (node->data == (gpointer) part)
-			break;
-		node = node->next;
-	}
-	
-	if (node == NULL) {
-		d(g_warning ("multipart_remove_part: %p does not seem to be a subpart of %p",
-			     part, multipart));
-		return;
-	}
-	
-	if (node == multipart->subparts) {
-		if (node->next)
-			node->next->prev = NULL;
-		multipart->subparts = node->next;
-	} else {
-		if (node->next)
-			node->next->prev = node->prev;
-		node->prev->next = node->next;
-	}
-	
-	g_list_free_1 (node);
+	if (!g_ptr_array_remove (multipart->children, part))
+		return FALSE;
 	
 	g_object_unref (part);
+	
+	return TRUE;
 }
 
 
@@ -520,14 +509,16 @@ multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part)
  * @part: mime part
  *
  * Removes the specified mime part from the multipart.
+ *
+ * Returns: %TRUE if the part was removed or %FALSE otherwise.
  **/
-void
+gboolean
 g_mime_multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part)
 {
 	g_return_if_fail (GMIME_IS_MULTIPART (multipart));
 	g_return_if_fail (GMIME_IS_OBJECT (part));
 	
-	GMIME_MULTIPART_GET_CLASS (multipart)->remove_part (multipart, part);
+	return GMIME_MULTIPART_GET_CLASS (multipart)->remove_part (multipart, part);
 }
 
 
@@ -535,26 +526,13 @@ static GMimeObject *
 multipart_remove_part_at (GMimeMultipart *multipart, int index)
 {
 	GMimeObject *part;
-	GList *node;
 	
-	if (!(node = g_list_nth (multipart->subparts, index))) {
-		d(g_warning ("multipart_remove_part_at: no part at index %d within %p", index, multipart));
+	if (index >= multipart->children->len)
 		return NULL;
-	}
 	
-	part = node->data;
+	part = multipart->children->pdata[index];
 	
-	if (node == multipart->subparts) {
-		if (node->next)
-			node->next->prev = NULL;
-		multipart->subparts = node->next;
-	} else {
-		if (node->next)
-			node->next->prev = node->prev;
-		node->prev->next = node->next;
-	}
-	
-	g_list_free_1 (node);
+	g_ptr_array_remove_index (multipart->children, index);
 	
 	return part;
 }
@@ -567,7 +545,7 @@ multipart_remove_part_at (GMimeMultipart *multipart, int index)
  *
  * Removes the mime part at position @index from the multipart.
  *
- * Returns the mime part that was removed.
+ * Returns: the mime part that was removed.
  **/
 GMimeObject *
 g_mime_multipart_remove_part_at (GMimeMultipart *multipart, int index)
@@ -583,15 +561,11 @@ static GMimeObject *
 multipart_get_part (GMimeMultipart *multipart, int index)
 {
 	GMimeObject *part;
-	GList *node;
 	
-	if (!(node = g_list_nth (multipart->subparts, index))) {
-		d(g_warning ("multipart_get_part: no part at index %d within %p", index, multipart));
+	if (index >= multipart->children->len)
 		return NULL;
-	}
-	
-	part = node->data;
 	
+	part = multipart->children->pdata[index];
 	g_object_ref (part);
 	
 	return part;
@@ -605,7 +579,7 @@ multipart_get_part (GMimeMultipart *multipart, int index)
  *
  * Gets the mime part at position @index within the multipart.
  *
- * Returns the mime part at position @index.
+ * Returns: the mime part at position @index.
  **/
 GMimeObject *
 g_mime_multipart_get_part (GMimeMultipart *multipart, int index)
@@ -620,10 +594,7 @@ g_mime_multipart_get_part (GMimeMultipart *multipart, int index)
 static int
 multipart_get_number (GMimeMultipart *multipart)
 {
-	if (!multipart->subparts)
-		return 0;
-	
-	return g_list_length (multipart->subparts);
+	return multipart->children->len;
 }
 
 
@@ -633,7 +604,7 @@ multipart_get_number (GMimeMultipart *multipart)
  *
  * Gets the number of mime parts contained within the multipart.
  *
- * Returns the number of mime parts contained within the multipart.
+ * Returns: the number of mime parts contained within the multipart.
  **/
 int
 g_mime_multipart_get_number (GMimeMultipart *multipart)
@@ -685,7 +656,7 @@ multipart_set_boundary (GMimeMultipart *multipart, const char *boundary)
 		
 		strcpy (bbuf, "=-");
 		p = (unsigned char *) bbuf + 2;
-		p += g_mime_utils_base64_encode_step (digest, 16, p, &state, &save);
+		p += g_mime_encoding_base64_encode_step (digest, 16, p, &state, &save);
 		*p = '\0';
 		
 		boundary = bbuf;
@@ -733,7 +704,7 @@ multipart_get_boundary (GMimeMultipart *multipart)
  * %NULL, then an auto-generated boundary will be set on the multipart
  * and returned.
  *
- * Returns the boundary on the multipart.
+ * Returns: the boundary on the multipart.
  **/
 const char *
 g_mime_multipart_get_boundary (GMimeMultipart *multipart)
@@ -755,18 +726,15 @@ g_mime_multipart_get_boundary (GMimeMultipart *multipart)
 void
 g_mime_multipart_foreach (GMimeMultipart *multipart, GMimePartFunc callback, gpointer user_data)
 {
-	GList *subpart;
+	GMimeObject *part;
+	guint i;
 	
 	g_return_if_fail (GMIME_IS_MULTIPART (multipart));
 	g_return_if_fail (callback != NULL);
 	
-	subpart = multipart->subparts;
-	while (subpart) {
-		GMimeObject *part = subpart->data;
-		
+	for (i = 0; i < multipart->children->len; i++) {
+		part = multipart->children->pdata[i];
 		callback (part, user_data);
-		
-		subpart = subpart->next;
 	}
 }
 
@@ -779,14 +747,16 @@ g_mime_multipart_foreach (GMimeMultipart *multipart, GMimePartFunc callback, gpo
  * Gets the mime part with the content-id @content_id from the
  * multipart @multipart.
  *
- * Returns the GMimeObject whose content-id matches the search string,
+ * Returns: the GMimeObject whose content-id matches the search string,
  * or %NULL if a match cannot be found.
  **/
 GMimeObject *
 g_mime_multipart_get_subpart_from_content_id (GMimeMultipart *multipart, const char *content_id)
 {
 	GMimeObject *object = (GMimeObject *) multipart;
-	GList *subparts;
+	GMimeObject *subpart, *part;
+	GMimeMultipart *mpart;
+	guint i;
 	
 	g_return_val_if_fail (GMIME_IS_MULTIPART (multipart), NULL);
 	g_return_val_if_fail (content_id != NULL, NULL);
@@ -796,24 +766,17 @@ g_mime_multipart_get_subpart_from_content_id (GMimeMultipart *multipart, const c
 		return object;
 	}
 	
-	subparts = multipart->subparts;
-	while (subparts) {
-		GMimeObject *part = NULL;
-		GMimeObject *subpart;
-		
-		subpart = subparts->data;
+	for (i = 0; i < multipart->children->len; i++) {
+		subpart = multipart->children->pdata[i];
 		
 		if (GMIME_IS_MULTIPART (subpart)) {
-			part = g_mime_multipart_get_subpart_from_content_id (GMIME_MULTIPART (subpart), content_id);
+			mpart = (GMimeMultipart *) subpart;
+			if ((part = g_mime_multipart_get_subpart_from_content_id (mpart, content_id)))
+				return part;
 		} else if (subpart->content_id && !strcmp (subpart->content_id, content_id)) {
 			g_object_ref (subpart);
-			part = subpart;
+			return subpart;
 		}
-		
-		if (part)
-			return part;
-		
-		subparts = subparts->next;
 	}
 	
 	return NULL;
